package com.romhay.chatter_sms

import android.Manifest
import android.app.Activity
import android.content.pm.PackageManager
import android.database.Cursor
import android.net.Uri
import com.romhay.chatter.permissions.Permissions
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.PluginRegistry
import org.json.JSONException
import org.json.JSONObject

enum class SmsQueryRequest {
    Inbox, Sent, Draft;

    fun toUri(): Uri {
        return when {
            this == Inbox -> {
                Uri.parse("content://sms/inbox")
            }
            this == Sent -> {
                Uri.parse("content://sms/sent")
            }
            else -> {
                Uri.parse("content://sms/draft")
            }
        }
    }
}

class SMSQueryHandler(private val result: MethodChannel.Result, private var request: SmsQueryRequest, private val activity: Activity,
                      start: Int, count: Int, threadId: Int, address: String?) : PluginRegistry.RequestPermissionsResultListener {
    private val permissionsList = arrayOf(Manifest.permission.READ_SMS)
    private var start = 0
    private var count = -1
    private var threadId = -1
    private var address: String? = null

    fun handle() {
        if (Permissions(activity).checkAndRequestPermission(permissionsList, Permissions.SEND_SMS_ID_REQ)) {
            querySms()
        }
    }

    private fun readSms(cursor: Cursor): JSONObject {
        val res = JSONObject()
        for (idx in 0 until cursor.columnCount) {
            try {
                if (cursor.getColumnName(idx) == "address" || cursor.getColumnName(idx) == "body") {
                    res.put(cursor.getColumnName(idx), cursor.getString(idx))
                } else if (cursor.getColumnName(idx) == "date" || cursor.getColumnName(idx) == "date_sent") {
                    res.put(cursor.getColumnName(idx), cursor.getLong(idx))
                } else {
                    res.put(cursor.getColumnName(idx), cursor.getInt(idx))
                }
            } catch (e: JSONException) {
                e.printStackTrace()
            }
        }
        return res
    }

    private fun querySms() {
        val list: ArrayList<JSONObject> = ArrayList()
        val cursor: Cursor = activity.contentResolver.query(this.request.toUri(), null, null, null, null)!!
        if (!cursor.moveToFirst()) {
            cursor.close()
            result.success(list)
            return
        }
        while (cursor.moveToNext() && count != 0) {
            val obj = readSms(cursor)
            try {
                if (threadId >= 0 && obj.getInt("thread_id") != threadId) {
                    continue
                }
                if (address != null && obj.getString("address") != address) {
                    continue
                }
            } catch (e: JSONException) {
                e.printStackTrace()
            }
            if (start > 0) {
                start--
                continue
            }
            list.add(obj)
            if (count > 0) {
                count--
            }
        }
        cursor.close()
        result.success(list)
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<out String>?, grantResults: IntArray?): Boolean {
        if (requestCode != Permissions.READ_SMS_ID_REQ) {
            return false
        }
        var isOk = true
        for (res in grantResults!!) {
            if (res != PackageManager.PERMISSION_GRANTED) {
                isOk = false
                break
            }
        }
        if (isOk) {
            querySms()
            return true
        }
        result.error("#01", "permission denied", null)
        return false
    }

    init {
        this.start = start
        this.count = count
        this.threadId = threadId
        this.address = address
    }
}

class SMSQuery(private val registrar: PluginRegistry.Registrar) : MethodChannel.MethodCallHandler {
    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        var start = 0
        var count = -1
        var threadId = -1
        var address: String? = null
        val request: SmsQueryRequest = when (call.method) {
            "getInbox" -> SmsQueryRequest.Inbox
            "getSent" -> SmsQueryRequest.Sent
            "getDraft" -> SmsQueryRequest.Draft
            else -> {
                result.notImplemented()
                return
            }
        }

        if (call.hasArgument("start")) {
            start = call.argument("start")!!
        }
        if (call.hasArgument("count")) {
            count = call.argument("count")!!
        }
        if (call.hasArgument("thread_id")) {
            threadId = call.argument("thread_id")!!
        }
        if (call.hasArgument("address")) {
            address = call.argument("address")
        }

        val handler = SMSQueryHandler(result, request, registrar.activity(), start, count, threadId, address)
        registrar.addRequestPermissionsResultListener(handler)
        handler.handle()
    }
}